package net.wanji.datacenter.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.datacenter.constant.EventAbnormalEnum;
import net.wanji.datacenter.mapper.EventInfoMapper;
import net.wanji.datacenter.pojo.dto.EventDataDTO;
import net.wanji.datacenter.pojo.po.EventInfoPO;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2024/11/26 15:43
 */
@Service
@Slf4j
public class EventDataServiceImpl implements EventDataService {

    @Resource
    private EventInfoMapper eventInfoMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void save(String originData) {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            EventDataDTO eventDataDTO = mapper.readValue(originData, EventDataDTO.class);
            String eventId = eventDataDTO.getEventId();
            // 如果同一时间id更新结束时间和告警状态
            if (StringUtils.isNotBlank(eventId)) {
                EventInfoPO eventInfoPO4DB = eventInfoMapper.selectByEventSerialNumber(eventId);
                if (Objects.nonNull(eventInfoPO4DB)) {
                    eventInfoPO4DB.setModifyTime(new Date());
                    eventInfoPO4DB.setEndTime(eventDataDTO.getEventEndTime());
                    eventInfoPO4DB.setAlarmStatus(eventDataDTO.getEventStatus());
                    eventInfoMapper.updateOne(eventInfoPO4DB);
                } else {
                    EventInfoPO eventInfoPO = getEventInfoPO(eventDataDTO);
                    eventInfoMapper.insertOne(eventInfoPO);
                }
            }
        } catch (Exception e) {
            log.error("事件保存异常：", e);
            throw new RuntimeException(e);
        }
    }

    @NotNull
    private static EventInfoPO getEventInfoPO(EventDataDTO eventDataDTO) {
        EventInfoPO eventInfoPO = new EventInfoPO();
        String eventTypeName = eventDataDTO.getEventType();
        setEventType(eventInfoPO, eventTypeName);
        Date eventStartTime = eventDataDTO.getEventStartTime();
        eventInfoPO.setStartTime(eventStartTime);
        eventInfoPO.setEndTime(eventDataDTO.getEventEndTime());
        eventInfoPO.setEventSerialNumber(eventDataDTO.getEventId());
        eventInfoPO.setCrossId(eventDataDTO.getCrossId());
        eventInfoPO.setGreenId(eventDataDTO.getGreenId());
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
        String dateStr = format.format(eventStartTime);
        eventInfoPO.setDt(Integer.valueOf(dateStr));
        eventInfoPO.setLat(eventInfoPO.getLat());
        eventInfoPO.setLng(eventInfoPO.getLng());
        eventInfoPO.setCategory(eventDataDTO.getEventLevel());
        eventInfoPO.setPlaceDesc(eventDataDTO.getEventType());
        eventInfoPO.setDir(eventDataDTO.getDirection());
        eventInfoPO.setRuksj(new Date());
        eventInfoPO.setAlarmStatus(eventDataDTO.getEventStatus());
        return eventInfoPO;
    }

    private static void setEventType(EventInfoPO eventInfoPO, String eventTypeName) {
        if (EventAbnormalEnum.PHASE_EMPTY.getDesc().contains(eventTypeName)) {
            eventInfoPO.setType(EventAbnormalEnum.PHASE_EMPTY.getType());
        }
        if (EventAbnormalEnum.CROSS_UNBALANCE.getDesc().contains(eventTypeName)) {
            eventInfoPO.setType(EventAbnormalEnum.CROSS_UNBALANCE.getType());
        }
        if (EventAbnormalEnum.CROSS_OVERFLOW.getDesc().contains(eventTypeName)) {
            eventInfoPO.setType(EventAbnormalEnum.CROSS_OVERFLOW.getType());
        }
        if (EventAbnormalEnum.GREEN_SLOW.getDesc().contains(eventTypeName)) {
            eventInfoPO.setType(EventAbnormalEnum.GREEN_SLOW.getType());
        }
        if (EventAbnormalEnum.GREEN_CONGEST.getDesc().contains(eventTypeName)) {
            eventInfoPO.setType(EventAbnormalEnum.GREEN_CONGEST.getType());
        }
        if (EventAbnormalEnum.CROSS_CONGEST.getDesc().contains(eventTypeName)) {
            eventInfoPO.setType(EventAbnormalEnum.CROSS_CONGEST.getType());
        }
    }
}
